﻿using System;
using System.IO;
using System.Text;
using System.Threading.Tasks;
using WingRtmpService.ChannelManagers.Messages;
using WingRtmpService.ChannelManagers.Netease.Helper;
using WingServerCommon.Log;

namespace WingRtmpService.ChannelManagers.Ali
{
     class AliChannelManager : IChannelManager
    {
        private string _protocol;//网址协议 http or https
        private string _playDomain;//拉流域名 
        private string _pushDomain;//推流域名
        private string _appName;
        private readonly DateTime _unixStarTime = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
        public AliChannelManager(string useSSL,string playDomain, string pushDomain,string appName)
        {
            _appName = appName;
            _protocol = useSSL == "true" ? "https:" : "http:"; 
            _playDomain = playDomain;
            _pushDomain = pushDomain;
        }
        private long GetUnixTime(DateTime time)
        {
            return (long)(time.ToUniversalTime() - _unixStarTime).TotalSeconds;
        }

        private string GetPushUrl(String streamName, long time)
        {
            return $"rtmp://{_pushDomain}/{_appName}/{streamName}";
        }
        private string GetPullUrl(string streamName)
        {
            return $"rtmp://{_playDomain}/{_appName}/{streamName}";
        }

        private string GetHttpPullUrl(string streamName)
        {
            return $"{_protocol}//{_playDomain}/{_appName}/{streamName}.flv";
        }

        private string GetHlsPullUrl(string streamName)
        {
            return $"{_protocol}//{_playDomain}/{_appName}/{streamName}.m3u8";
        }

        public ICreateChannelResult CreateChannel(ChannelParameter channel)
        {
            if (channel.Parameter.ContainsKey(ChannelConstant.CreateChannelKey))
            {
                var streamName = channel.Parameter[ChannelConstant.CreateChannelKey];

                var channelInfo = new ChannelInfo
                {
                    Cid = channel.Parameter[ChannelConstant.CreateChannelKey],
                    PushUrl = GetPushUrl(streamName, GetUnixTime(DateTime.Now.Date.AddDays(7))),
                    RtmpPullUrl = GetPullUrl(streamName),
                    HttpPullUrl = GetHttpPullUrl(streamName),
                    HlsPullUrl = GetHlsPullUrl(streamName)
                };
                return new CreateChannelResult
                {
                    Code = (int)SendResultStatus.Success,
                    ChannelInfo = channelInfo
                };
            }

            Logger.WriteLineError("Create channel parameter is empty.");
            throw new InvalidDataException();
        }

        public Task<IDeleteChannelResult> DeleteChannelAsync(ChannelParameter channel)
        {
            return Task.FromResult<IDeleteChannelResult>(new DeleteChannelResult { Code = (int)SendResultStatus.Success });
        }
    }
}
