﻿using System.IO;
using System.Threading.Tasks;
using WingRtmpService.ChannelManagers.Messages;
using WingRtmpService.ChannelManagers.Netease.Helper;
using WingServerCommon.Log;

namespace WingRtmpService.ChannelManagers.Common
{
    class CommonChannelManager : IChannelManager
    {
        private readonly string _pushUrl;
        private readonly string _playUrl;
        public CommonChannelManager(string pushUrl, string playUrl)
        {
            var rtmpPrefix = pushUrl.Replace("http://", "").Replace("https://", "");
            _pushUrl = $"rtmp://{rtmpPrefix}";
            _playUrl = playUrl;
        }

        public ICreateChannelResult CreateChannel(ChannelParameter channel)
        {
            if (channel.Parameter.ContainsKey(ChannelConstant.CreateChannelKey))
            {
                //SendResultStatus
                var liveUrl = string.Empty;
                if (!string.IsNullOrEmpty(_pushUrl))
                {
                    liveUrl = _pushUrl + channel.Parameter[ChannelConstant.CreateChannelKey];
                }
                var httpUrl = string.Empty;
                if (!string.IsNullOrEmpty(_playUrl))
                {
                    httpUrl = _playUrl + channel.Parameter[ChannelConstant.CreateChannelKey];
                }
                var channelInfo = new ChannelInfo
                {
                    Cid = channel.Parameter[ChannelConstant.CreateChannelKey],
                    PushUrl = liveUrl,
                    RtmpPullUrl = liveUrl,
                    HttpPullUrl = httpUrl + ".flv",
                    HlsPullUrl = httpUrl + "/index.m3u8",
                };
                return new CreateChannelResult
                {
                    Code = (int)SendResultStatus.Success,
                    ChannelInfo = channelInfo
                };
            }

            Logger.WriteLineError("Create channel parameter is empty.");
            throw new InvalidDataException();
        }

        public Task<IDeleteChannelResult> DeleteChannelAsync(ChannelParameter channel)
        {
            return Task.FromResult<IDeleteChannelResult>(new DeleteChannelResult { Code = (int)SendResultStatus.Success });
        }
    }
}
