﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using WingServerCommon.Log;
using WingServerCommon.Config;

namespace WingRtmpService.ChannelManagers.Netease.Helper
{
    public class HttpClientBuilder
    {
        private readonly string _appKey;

        private readonly string _appSecret;

        private const int TimeOut = 30000;

        private readonly bool _forceConstantCheckSum;

        private readonly string _constantCurrentTime;

        private readonly string _constantCheckSum;

        public HttpClientBuilder(string appKey, string appSecret, bool forceConstantCheckSum, string constantCurrentTime, string constantCheckSum)
        {
            _forceConstantCheckSum = forceConstantCheckSum;
            _constantCurrentTime = constantCurrentTime;
            _constantCheckSum = constantCheckSum;
            _appKey = appKey;
            _appSecret = appSecret;
        }

        public string HttpPost(string url, string json)
        {
            var request = CreateWebRequest(url, "POST");

            try
            {
                using (var streamWriter = new StreamWriter(request.GetRequestStream()))
                {
                    streamWriter.Write(json);
                    streamWriter.Flush();
                    streamWriter.Close();
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"send data to ne exception: {ex}");
            }
          
            var responseFromServer = string.Empty;
            WebResponse response = null;
            Stream dataStream = null;
            try
            {
                response = (HttpWebResponse) request.GetResponse();
                dataStream = response.GetResponseStream();
                if (dataStream != null)
                {
                    using (var reader = new StreamReader(dataStream))
                    {
                        responseFromServer = reader.ReadToEnd();
                    }                    
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"Get response from ne exception: {ex}");
            }
            finally
            {
                dataStream?.Close();
                response?.Close();
            }

            return responseFromServer;
        }

        public WebRequest CreateWebRequest(string url, string method)
        {
            var request = WebRequest.Create(url);
            var appKey = _appKey;
            var appSecret = _appSecret;
            var nonce = "1";//TO开发者：具体参考开发文档

            var ts = DateTime.Now.ToUniversalTime() - new DateTime(1970, 1, 1);
            var ticks = Convert.ToInt32(ts.TotalSeconds);
            IDictionary<object, string> headers = new Dictionary<object, string>();
            if (_forceConstantCheckSum)
            {
                headers["CurTime"] = _constantCurrentTime;
                headers["CheckSum"] = _constantCheckSum;
            }
            else
            {
                var curTime = ticks.ToString();
                var checkSum = CheckSumBuilder.GetCheckSum(appSecret, nonce, curTime);
                headers["CurTime"] = curTime;
                headers["CheckSum"] = checkSum;
            }
           
            headers["AppKey"] = appKey;
            headers["Nonce"] = nonce;
            headers["ContentType"] = "application/json;charset=utf-8";

            request.ContentType = "application/json";
            request.Method = method;

            request.Timeout = TimeOut;

            foreach (var v in headers)
            {
                if (v.Key is HttpRequestHeader)
                    request.Headers[(HttpRequestHeader)v.Key] = v.Value;
                else
                    request.Headers[v.Key.ToString()] = v.Value;
            }

            return request;
        }
    }
}
