﻿using System;
using System.IO;
using System.Text;
using System.Threading.Tasks;
using WingRtmpService.ChannelManagers.Messages;
using WingRtmpService.ChannelManagers.Netease.Helper;
using WingServerCommon.Log;
using WingServerCommon.Config;
using WingServerCommon.Utilities;
using WingServerCommon.Config.Parameters;

namespace WingRtmpService.ChannelManagers.Tencent
{
    class TxChannelManager : IChannelManager
    {
        private string _playDomain;//拉流域名 
        private string _pushDomain;//推流域名
        private string _pushKey;
        private readonly DateTime _unixStarTime = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);

        public TxChannelManager()
        {
            _playDomain = EnvironmentConfigs.Rtmp.RTMPPlayUrl.TrimEnd('/');
            _pushDomain = EnvironmentConfigs.Rtmp.RTMPPushUrl.TrimEnd('/');
            _pushKey = EnvironmentConfigManager.GetParammeter<StringParameter>("Rtc", "RTCPushKey").Value;
        }

        private long GetUnixTime(DateTime time)
        {
            return (long)(time.ToUniversalTime() - _unixStarTime).TotalSeconds;
        }

        private string GetPushUrl(String streamName, long time)
        {
            var txTime = time.ToString("X");
            var input = new StringBuilder();
            input.Append(_pushKey);
            input.Append(streamName).Append(txTime);

            var txSecret = MD5.GetHashString(Encoding.UTF8.GetBytes(input.ToString())).ToLower();
            var parameters = string.IsNullOrEmpty(txSecret)
                ? string.Empty
                : new StringBuilder().Append("txSecret=").Append(txSecret).Append("&").Append("txTime=").Append(txTime)
                    .ToString();
            var rtmpPrefix = _pushDomain.Replace("http://", "").Replace("https://", "");
            return $"rtmp://{rtmpPrefix}/{streamName}?{parameters}";
        }

        private string GetPullUrl(string streamName)
        {
            var rtmpPrefix = _playDomain.Replace("http://", "").Replace("https://", "");
            return $"rtmp://{rtmpPrefix}/{streamName}";
        }

        private string GetHttpPullUrl(string streamName)
        {
            return $"{_playDomain}/{streamName}.flv";
        }

        private string GetHlsPullUrl(string streamName)
        {
            return $"{_playDomain}/{streamName}.m3u8";
        }

        public ICreateChannelResult CreateChannel(ChannelParameter channel)
        {
            if (channel.Parameter.ContainsKey(ChannelConstant.CreateChannelKey))
            {
                var streamName = channel.Parameter[ChannelConstant.CreateChannelKey];

                var channelInfo = new ChannelInfo
                {
                    Cid = channel.Parameter[ChannelConstant.CreateChannelKey],
                    PushUrl = GetPushUrl(streamName, GetUnixTime(DateTime.Now.Date.AddDays(7))),
                    RtmpPullUrl = GetPullUrl(streamName),
                    HttpPullUrl = GetHttpPullUrl(streamName),
                    HlsPullUrl = GetHlsPullUrl(streamName)
                };
                return new CreateChannelResult
                {
                    Code = (int)SendResultStatus.Success,
                    ChannelInfo = channelInfo
                };
            }

            Logger.WriteLineError("Create channel parameter is empty.");
            throw new InvalidDataException();
        }

        public Task<IDeleteChannelResult> DeleteChannelAsync(ChannelParameter channel)
        {
            return Task.FromResult<IDeleteChannelResult>(new DeleteChannelResult { Code = (int)SendResultStatus.Success });
        }
    }
}
