﻿using System;
using System.Collections.Concurrent;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace WingRtmpService.EmbeddedRtmpServer.IO
{
    public delegate object ConstructorCall();

    static class ObjectCreator
    {
        private static readonly ConcurrentDictionary<Type, ConstructorCall> ConstructorCache = new ConcurrentDictionary<Type, ConstructorCall>();

        private static ConstructorCall CompileObjectConstructor(ConstructorInfo constructor)
        {
            var expression = Expression.New(constructor);
            var lambda = Expression.Lambda<ConstructorCall>(expression);
            return lambda.Compile();
        }

        public static bool IsRegistered(this Type type)
        {
            return ConstructorCache.ContainsKey(type);
        }

        public static void Register(this Type type)
        {
            ConstructorCache.AddOrUpdate(type, 
                t =>
                {
                    var constructor = t.GetConstructors().FirstOrDefault(x => x.GetParameters().Length == 0);
                    if (constructor == null)
                        throw new ArgumentException("Type does not have any accessible parameterless constructors.");
                    return CompileObjectConstructor(constructor);
                },
                (t, ec) =>
                {
                    var constructor = t.GetConstructors().FirstOrDefault(x => x.GetParameters().Length == 0);
                    if (constructor == null)
                        throw new ArgumentException("Type does not have any accessible parameterless constructors.");
                    return CompileObjectConstructor(constructor);
                }
            );
        }

        public static object CreateInstance(this Type type)
        {
            var creator = ConstructorCache.GetOrAdd(type, _ =>
            {
                var constructor = type.GetConstructors().FirstOrDefault(x => x.GetParameters().Length == 0);
                if (constructor == null)
                    throw new ArgumentException("Type does not have any accessible parameterless constructors.");
                return CompileObjectConstructor(constructor);
            });
            return creator();
        }
    }
}
