﻿using System;
using System.Collections.Generic;
using WingRtmpService.EmbeddedRtmpServer.IO.AMF3;
using WingRtmpService.EmbeddedRtmpServer.IO.ObjectWrappers;

namespace WingRtmpService.EmbeddedRtmpServer.IO
{
    class ObjectWrapperFactory
    {
        private static readonly string ExternalizableTypeName = typeof(IExternalizable).FullName;

        private readonly SerializationContext _context;
        private readonly IObjectWrapper _defaultWrapper;
        private readonly Dictionary<Type, IObjectWrapper> _wrappers = new Dictionary<Type, IObjectWrapper>();
        
        public ObjectWrapperFactory(SerializationContext context)
        {
            this._context = context;
            
            _defaultWrapper = new BasicObjectWrapper(context);

            _wrappers[typeof(AsObject)] = new AsObjectWrapper(context);
            _wrappers[typeof(IExternalizable)] = new ExternalizableWrapper(context);
            _wrappers[typeof(Exception)] = new ExceptionWrapper(context);
        }

        public IObjectWrapper GetInstance(Type type)
        {
            if (type.GetInterface(ExternalizableTypeName, true) != null)
                return _wrappers[typeof(IExternalizable)];

            if (_wrappers.TryGetValue(type, out var wrapper))
                return wrapper;

            foreach (var entry in _wrappers)
            {
                if (type.IsSubclassOf(entry.Key))
                    return entry.Value;
            }
            
            return _defaultWrapper;
        }

        public ClassDescription GetClassDescription(object obj)
        {
            var instance = GetInstance(obj.GetType());
            return instance.GetClassDescription(obj);
        }

        public ClassDescription GetClassDescription(Type type, object obj)
        {
            var instance = GetInstance(type);
            return instance.GetClassDescription(obj);
        }
    }
}
