﻿using System;

namespace WingRtmpService.EmbeddedRtmpServer.IO
{
    public class SerializationContext
    {
        // Specifies the course of action to take when a type cannot be serialized to a typed
        // object (because it has not been registered)
        private readonly FallbackStrategy _fallbackStrategy;
        private readonly SerializerObjectFactory _serializerObjectFactory;
        private readonly ObjectWrapperFactory _objectWrapperFactory;

        public static readonly SerializationContext Default = new SerializationContext();


        public SerializationContext()
        {
            _fallbackStrategy = FallbackStrategy.DynamicObject;
            _serializerObjectFactory = new SerializerObjectFactory();
            _objectWrapperFactory = new ObjectWrapperFactory(this);
        }


        internal string GetAlias(string typeName) => _serializerObjectFactory.GetAlias(typeName);

        internal bool CanCreate(string typeName) => _serializerObjectFactory.CanCreate(typeName);

        internal object Create(string typeName) => _serializerObjectFactory.Create(typeName);

        internal ClassDescription GetClassDescription(object obj) => _objectWrapperFactory.GetClassDescription(obj);

        internal ClassDescription GetClassDescription(Type type, object obj) => _objectWrapperFactory.GetClassDescription(type, obj);


        internal DeserializationStrategy GetDeserializationStrategy(string typeName)
            => CanCreate(typeName)
                ? DeserializationStrategy.TypedObject
                : GetFallbackDeserializationStrategy();


        DeserializationStrategy GetFallbackDeserializationStrategy()
             => _fallbackStrategy == FallbackStrategy.DynamicObject
                ? DeserializationStrategy.DynamicObject
                : DeserializationStrategy.Exception;
    }
}
