﻿using System.IO;
using WingRtmpService.EmbeddedRtmpServer.IO;
using WingRtmpService.EmbeddedRtmpServer.Messaging.Events;

namespace WingRtmpService.EmbeddedRtmpServer.Net
{
    class FlvTag
    {
        public static readonly byte[] FlvHeader = { 0x46, 0x4c, 0x56, 0x01, 0x00, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00, 0x00 };

        public byte[] TagData { get; }

        public FlvTag(ByteData mediaPacket)
        {
            var previousTagSize = 11 + mediaPacket.Header.PacketLength;
            //The last 4 bytes is the previous tag size.
            var tagBuffer = new byte[previousTagSize + 4];
            using (var ms = new MemoryStream(tagBuffer))
            {
                ms.Position = 0;
                using (var writer = new AmfWriter(ms, SerializationContext.Default))
                {
                    writer.WriteByte((byte) mediaPacket.Header.MessageType);
                    writer.WriteUInt24(mediaPacket.Header.PacketLength);
                    writer.WriteUInt24(mediaPacket.Header.Timestamp);
                    writer.WriteReverseInt(0);
                    writer.Write(mediaPacket.Data);
                    writer.WriteUInt32((uint)previousTagSize);
                    TagData = ms.ToArray();
                }
            }
        }
    }
}
