﻿using System;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading;
using WingRtmpService.EmbeddedRtmpServer.Log;

namespace WingRtmpService.EmbeddedRtmpServer.Net.Http
{
    class HlsProducer:IDisposable
    {
        private readonly string _cacheFolder;
        private readonly int _processId;
        private readonly Process _process;

        public HlsProducer(string streamSource, string cacheFolder)
        {
            _cacheFolder = cacheFolder;
            if (!Directory.Exists(_cacheFolder))
            {
                Directory.CreateDirectory(_cacheFolder);
            }
            var streamIndex = Path.Combine(_cacheFolder, "play.m3u8");
            var args = $"-v verbose -i {streamSource} -c:v copy -c:a aac -hls_time 3 -hls_list_size 5 -hls_wrap 5 -start_number 1 {streamIndex}";
            _process = Process.Start(new ProcessStartInfo(RtmpServerConfig.FFmpegPath)
            {
                Arguments = args,
                CreateNoWindow = true,
                WindowStyle = ProcessWindowStyle.Hidden
            });
            if (_process == null)
            {
                throw new InvalidOperationException("Could not start HLS transcoder.");
            }
            _processId = _process.Id;
        }

        public void Dispose()
        {
            try
            {
                _process?.Kill();
            }
            catch(Exception ex)
            {
                Logger.WriteLineWarn($"Close HLS transcoder failed. error:{ex.Message}");
            }
            //Wait for the process exit.
            while (Process.GetProcesses().Any(x=>x.Id == _processId))
            {
                Thread.Sleep(1);
            }
            Directory.Delete(_cacheFolder, true);
        }
    }
}
