﻿using System;
using WingRtmpService.EmbeddedRtmpServer.Messaging.Events;

namespace WingRtmpService.EmbeddedRtmpServer.Net
{
    class RtmpPlayer: IPlayer
    {
        private readonly RtmpConnection _connection;

        public event EventHandler Closed;

        public string ChannelName { get; }

        public bool IsClosed { get; private set; }

        public RtmpPlayer(string channelName, RtmpConnection connection)
        {
            ChannelName = channelName;
            _connection = connection;
            _connection.ConnectionType = ConnectionType.Player;
            _connection.Closed += (sender, args) => OnClosed();
        }

        public void SendMetaData(VideoData videoConfigureRecord, AudioData audioConfigureRecord)
        {
            if (audioConfigureRecord != null)
            {
                _connection.SendAmf0Data(audioConfigureRecord);
            }

            if (videoConfigureRecord != null)
            {
                _connection.SendAmf0Data(videoConfigureRecord);
            }
        }

        public void Close()
        {
            _connection.Close();
            IsClosed = true;
        }

        public void SendVideoData(VideoData videoData)
        {
            _connection.SendAmf0Data(videoData);
        }

        public void SendAudioData(AudioData audioData)
        {
            _connection.SendAmf0Data(audioData);
        }

        private void OnClosed()
        {
            Closed?.Invoke(this, EventArgs.Empty);
        }
    }
}
