﻿using System;
using System.IO;

namespace WingRtmpService.EmbeddedRtmpServer.Net
{
    class RtmpServerConfig
    {
        /// <summary>
        /// The port which the rtmp server will use.
        /// </summary>
        public static int RtmpPort = 1935;

        /// <summary>
        /// The http port which the http extension will use.
        /// </summary>
        public static int HttpPort = 80;

        /// <summary>
        /// The default app for rtmp server.
        /// </summary>
        public static string DefaultApp = "live";

        /// <summary>
        /// This will increase the memory usage, be careful.
        /// </summary>
        public static bool EnablePublishVideoCache = true;

        /// <summary>
        /// This will increase the memory usage, be careful.
        /// </summary>
        public static int PublishVideoCacheGroupSize = 1;

        /// <summary>
        /// Polling mode if true, only one thread handle all connections, otherwise each connection will handle io operations in their own thread.
        /// </summary>
        public static bool EnableGlobalPollingMode = false;

        /// <summary>
        /// If true it will clean no operation connection when timeout, suggest false.
        /// </summary>
        public static bool AutoCleanNoOperationConnections = false;

        /// <summary>
        /// If true the player can continuous play(in buffering state) when the publisher is not online, otherwise, the player will will closed.
        /// </summary>
        public static bool AllowPlayWithoutPublish = true;


        /// <summary>
        /// If true, the http-flv protocol will be enabled.
        /// </summary>
        public static bool EnableHttpFlvExtension = false;

        /// <summary>
        /// If true, will record all publisher's live stream.
        /// </summary>
        public static bool EnableRecordLiveStream = false;

        /// <summary>
        /// Define the record folder for the recorded live stream.
        /// </summary>
        public static string RecordFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Records");

        /// <summary>
        /// If true the HLS protocol will be enabled.
        /// </summary>
        public static bool EnableHlsExtension = true;

        /// <summary>
        /// Define the cache folder of HLS.
        /// </summary>
        public static string HlsCacheFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "HLS");

        /// <summary>
        /// Define the ffmpeg path.
        /// </summary>
        public static string FFmpegPath;

        static RtmpServerConfig()
        {
            //Initialize the ffmpeg path
            switch (Environment.OSVersion.Platform)
            {
                case PlatformID.Win32S:
                    throw new NotSupportedException($"Not supported OS {Environment.OSVersion.Platform}");
                case PlatformID.Win32Windows:
                    throw new NotSupportedException($"Not supported OS {Environment.OSVersion.Platform}");
                case PlatformID.Win32NT:
                    FFmpegPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Packages", "ffmpeg.exe");
                    break;
                case PlatformID.WinCE:
                    throw new NotSupportedException($"Not supported OS {Environment.OSVersion.Platform}");
                case PlatformID.Unix:
                    FFmpegPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Packages", "ffmpeg");
                    break;
                case PlatformID.Xbox:
                    throw new NotSupportedException($"Not supported OS {Environment.OSVersion.Platform}");
                case PlatformID.MacOSX:
                    FFmpegPath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Packages", "ffmpeg");
                    break;
                default:
                    throw new NotSupportedException($"Not supported OS {Environment.OSVersion.Platform}");
            }
        }
    }
}
